import {customElement, property} from 'lit/decorators.js';
import {BaseSearchElement} from '../BaseSearchElement';
import {html, unsafeStatic} from 'lit/static-html.js';
import {repeat} from 'lit/directives/repeat.js';
import {TemplateResult} from 'lit';
import {SearchResultType} from '../types';

/**
 * A container for the search results. Handles rendering each result in its correct element.
 */
@customElement('search-results')
export class SearchResults extends BaseSearchElement {
  /**
   * An object keyed by the search result type and mapped to the element it should render as. I.e. {"node-page":"search-node-page-result"}. When using the display switcher types can be appended with the display id i.e `-grid` to make mapping specific to a display type.
   */
  @property({type: Object})
  elementMap: {[key: string]: string} = {
    default: 'search-result-element-default',
  };

  /**
   * The property to use for the result to map to an element.
   */
  @property()
  elementProp = 'elementType';

  /**
   * An object keyed by the search result type and mapped to the settings for it should pass to the element I.e. {"node-page":{"someSetting":"someValue"}}. When using the display switcher types can be appended with the display id i.e `-grid` to make mapping specific to a display type.
   */
  @property({type: Object})
  settings: {[key: string]: {}} = {};

  /**
   * Get the element type for the given result.
   * @param result
   */
  _mapResultElement(result: SearchResultType): string | undefined {
    let element = this.elementMap?.default;
    if (
      this.elementMap[
        `${result[this.elementProp]}-${this.context?.resultDisplay}`
      ]
    ) {
      element =
        this.elementMap[
          `${result[this.elementProp]}-${this.context?.resultDisplay}`
        ];
    } else if (this.elementMap[result[this.elementProp]]) {
      element = this.elementMap[result[this.elementProp]];
    }

    if (!element) {
      console.warn('No element mapping found for:', result);
      return undefined;
    }

    if (!customElements.get(element)) {
      console.warn(element + 'is not defined for element mapping:', result);
      return undefined;
    }

    return element;
  }

  /**
   * Get the settings for the given result.
   * @param result
   */
  _mapSettings(result: SearchResultType): {} {
    let settings = this.settings?.default ?? {};
    if (
      this.settings[
        `${result[this.elementProp]}-${this.context?.resultDisplay}`
      ]
    ) {
      settings =
        this.settings[
          `${result[this.elementProp]}-${this.context?.resultDisplay}`
        ];
    } else if (this.settings[result[this.elementProp]]) {
      settings = this.settings[result[this.elementProp]];
    }

    return settings;
  }

  /**
   * Get the result elements correctly mapped to the right element type.
   */
  _getResults(): TemplateResult | null {
    const results = this.context?.response?.search_results ?? [];

    if (results.length === 0) {
      return null;
    }

    return html`
      <div>
        <ul class="${this.context?.resultDisplay}">
          ${repeat(
            results,
            (result) => result.id,
            (result) => {
              const element = this._mapResultElement(result);

              if (!element) {
                return null;
              }

              return html`<li
                class="result-${this.safeIdentifier(result[this.elementProp])}"
              >
                <${unsafeStatic(element)}
                  .data=${result}
                  .settings=${this._mapSettings(result)}
                />
              </li>`;
            }
          )}
        </ul>
      </div>
    `;
  }

  override render() {
    return html`${this._getResults()} `;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'search-results': SearchResults;
  }
}
